@@ -11,7 +11,7 @@
 /***************************************************************/
 
 #include "config.h"
-static char const RCSID[] = "$Id: calendar.c,v 1.9 2000/02/18 03:45:43 dfs Exp $";
+static char const RCSID[] = "$Id: calendar.c,v 1.12 2005/04/12 01:49:45 dfs Exp $";
 
 #include <stdio.h>
 #include <string.h>
@@ -41,6 +41,8 @@ typedef struct cal_entry {
     char tag[TAG_LEN+1];
     char passthru[PASSTHRU_LEN+1];
     int duration;
+    char *filename;
+    int lineno;
 } CalEntry;
 
 /* Global variables */
@@ -88,7 +90,7 @@ void ProduceCalendar()
 
     if (CalMonths) {
 	FromJulian(JulianToday, &y, &m, &d);
-	JulianToday = Julian(y, m, 1);   
+	JulianToday = Julian(y, m, 1);
 	while (CalMonths--)
 	    DoCalendarOneMonth();
 	return;
@@ -148,9 +150,9 @@ static void DoCalendarOneWeek()
     PutChar('|');
     for (i=0; i<7; i++) {
 	FromJulian(OrigJul+i, &y, &m, &d);
-	sprintf(buf, "%d %c%c%c ", d, MonthName[m][0], MonthName[m][1], 
+	sprintf(buf, "%d %c%c%c ", d, MonthName[m][0], MonthName[m][1],
 		MonthName[m][2]);
-	if (OrigJul+i == RealToday)				  
+	if (OrigJul+i == RealToday)
 	    PrintLeft(buf, ColSpaces, '*');
 	else
 	    PrintLeft(buf, ColSpaces, ' ');
@@ -183,7 +185,7 @@ static void DoCalendarOneWeek()
 	PutChar('\n');
     }
 
-/* Write the final line */   
+/* Write the final line */
     WriteIntermediateCalLine();
 }
 
@@ -229,7 +231,7 @@ static void DoCalendarOneMonth()
 
     if (PsCal) printf("%s\n", PSEND);
     if (!DoSimpleCalendar) WriteCalTrailer();
-}   
+}
 
 /***************************************************************/
 /*                                                             */
@@ -271,7 +273,7 @@ static int WriteCalendarRow()
 	}
 	return (d+7-wd <= DaysInMonth(m, y));
     }
- 
+
 
 /* Here come the first few lines... */
     PutChar('|');
@@ -394,7 +396,7 @@ static int WriteOneCalLine()
     return done;
 }
 
-     
+
 /***************************************************************/
 /*                                                             */
 /*  WriteOneColLine                                            */
@@ -425,6 +427,7 @@ int col;
 	PrintLeft("", ColSpaces, ' ');
 	CalColumn[col] = e->next;
 	free(e->text);
+	free(e->filename);
 	free(e);
 	return 1;
     }
@@ -464,6 +467,7 @@ int col;
     if (!*s && !e->next) {
 	CalColumn[col] = e->next;
 	free(e->text);
+	free(e->filename);
 	free(e);
     } else {
 	e->pos = s;
@@ -527,7 +531,7 @@ int col;
 
 	    switch(tok.type) {
 
-            case T_Empty:
+	    case T_Empty:
 	    case T_Comment:
 		break;
 
@@ -544,7 +548,7 @@ int col;
 	    case T_UnSet:   r=DoUnset(&p);   break;
 	    case T_Clr:     r=DoClear(&p);   break;
 	    case T_Flush:   r=DoFlush(&p);   break;
-            case T_Debug:   break;  /* IGNORE DEBUG CMD */
+	    case T_Debug:   break;  /* IGNORE DEBUG CMD */
 	    case T_Dumpvars: break; /* IGNORE DUMPVARS CMD */
 	    case T_Banner:  break;  /* IGNORE BANNER CMD */
 	    case T_Omit:    r=DoOmit(&p);
@@ -556,7 +560,7 @@ int col;
 		break;
 	    case T_Pop:     r=PopOmitContext(&p);     break;
 	    case T_Push:    r=PushOmitContext(&p);    break;
-            case T_Preserve: r=DoPreserve(&p);        break;
+	    case T_Preserve: r=DoPreserve(&p);        break;
 	    case T_RemType: if (tok.val == RUN_TYPE) {
 		r=DoRun(&p);
 		break;
@@ -703,12 +707,12 @@ int col;
     if (!PsCal && trig.typ == PASSTHRU_TYPE) return OK;
 
     /* Remove any "at" times from PS or PSFILE reminders */
-    if (trig.typ == PASSTHRU_TYPE) {
+    if (trig.typ == PASSTHRU_TYPE && strcmp(trig.passthru, "COLOR")) {
 	tim.ttime = NO_TIME;
 	tim.duration = NO_TIME;
     }
 
-    /* If trigger date == today, add it to the current entry */   
+    /* If trigger date == today, add it to the current entry */
     DBufInit(&obuf);
     if (jul == JulianToday) {
 	NumTriggered++;
@@ -780,7 +784,17 @@ int col;
 	}
 	e->duration = tim.duration;
 	e->priority = trig.priority;
-	if (trig.typ == PASSTHRU_TYPE) {
+	e->filename = StrDup(FileName);
+	if(!e->filename) {
+	    free(e);
+	    return E_NO_MEM;
+	}
+	e->lineno = LineNo;
+
+	/* Ugly hack... a SPECIAL COLOR is not treated as "passthru"
+	   to preserve ordering and make it behave like a MSG-type reminder */
+
+	if (trig.typ == PASSTHRU_TYPE && strcmp(trig.passthru, "COLOR")) {
 	    StrnCpy(e->passthru, trig.passthru, PASSTHRU_LEN);
 	    e->pos = e->passthru;
 	    e->time = NO_TIME;
@@ -788,6 +802,11 @@ int col;
 	    CalPs[col] = e;
 	    SortCol(&CalPs[col]);
 	} else {
+	    if (trig.typ == PASSTHRU_TYPE) {
+		StrnCpy(e->passthru, trig.passthru, PASSTHRU_LEN);
+	    } else {
+		e->passthru[0] = 0;
+	    }
 	    e->pos = e->text;
 	    e->time = tim.ttime;
 	    e->next = CurCol;
@@ -819,6 +838,7 @@ int col, jul;
 /* Do all the PASSTHRU entries first, if any */
     FromJulian(jul, &y, &m, &d);
     while(e) {
+	if (DoPrefixLineNo) printf("# fileinfo %d %s\n", e->lineno, e->filename);
 	printf("%04d/%02d/%02d ", y, m+1, d);
 	printf("%s ", e->passthru);
 	printf("%s ", e->tag);
@@ -834,16 +854,23 @@ int col, jul;
 	}
 	printf("%s\n", e->text);
 	free(e->text);
+	free(e->filename);
 	n = e->next;
 	free(e);
 	e = n;
     }
     CalPs[col] = NULL;
 
-    e = CalColumn[col];				     
+    e = CalColumn[col];
     while(e) {
+	if (DoPrefixLineNo) printf("# fileinfo %d %s\n", e->lineno, e->filename);
 	printf("%04d/%02d/%02d", y, m+1, d);
-	printf(" * %s ", e->tag);
+	if (e->passthru[0]) {
+	    printf(" %s", e->passthru);
+	} else {
+	    printf(" *");
+	}
+	printf(" %s ", e->tag);
 	if (e->duration != NO_TIME) {
 	    printf("%d ", e->duration);
 	} else {
@@ -856,6 +883,7 @@ int col, jul;
 	}
 	printf("%s\n", e->text);
 	free(e->text);
+	free(e->filename);
 	n = e->next;
 	free(e);
 	e = n;
@@ -934,7 +962,7 @@ int tim;
     int h, min, hh;
 
     buf[0] = 0;
-   
+
     switch(ScFormat) {
 
     case SC_AMPM:
